package Sequencer.Sequencer;

import Bytes.Bytes.ByteArray;
import Bytes.Bytes.ByteArrayReference;
import DataStructures.Array.Structures.Array;
import DataStructures.Array.Structures.Data;
import ProcessingUnit.ProcessingUnit.ProcessingUnitStructure;
import ProcessingUnit.ProcessingUnitServer.ProcessingUnitServerStructure;

import static Bytes.Bytes.Bytes.FreeByteArray;
import static DataStructures.Array.Arrays.Arrays.CreateArray;
import static DataStructures.Array.Structures.Structures.*;
import static JSON.Writer.Writer.WriteJSON;
import static ProcessingUnit.ProcessingUnit.ProcessingUnit.*;
import static ProcessingUnit.ProcessingUnitServer.ProcessingUnitServer.*;
import static Unicode.Unicode.Unicode.UTF16ToUTF8;
import static arrays.arrays.arrays.CreateNumberArray;


public class Sequencer {
    public static boolean SequencerInitialize(ProcessingUnitServerStructure [] clients, ProcessingUnitServerStructure adminPort, ProcessingUnitStructure server, SequencerState state) {
        state.clientReqs = CreateNumberArray(clients.length, 0d);
        state.idles = 0d;
        state.idle = false;

        return true;
    }

    public static void SequencerIteration(ProcessingUnitServerStructure [] clients, ProcessingUnitServerStructure adminPort, ProcessingUnitStructure server, SequencerState state){
        ProcessingUnitServerStructure client;
        ByteArrayReference request, response;
        double i;
        boolean clientReq, adminPortReq;
        Data adminPortOutput;
        char [] adminPortOutputJSON;
        ByteArray adminPortOutputBytes;
        ByteArrayReference adminPortRequestBytes;
        Array clientReqsData;

        request = new ByteArrayReference();
        response = new ByteArrayReference();

        clientReq = false;
        adminPortReq = false;

        for(i = 0d; i < clients.length; i = i + 1d){
            client = clients[(int)i];

            if(ServerCheck(client)){
                state.clientReqs[(int)i] = state.clientReqs[(int)i] + 1d;
                clientReq = true;

                ServerReceive(client, request);
                Call(server, request.bytes, response);
                ServerSend(client, response.bytes);
            }
        }

        if(ServerCheck(adminPort)){
            adminPortReq = true;

            adminPortRequestBytes = new ByteArrayReference();
            ServerReceive(adminPort, adminPortRequestBytes);

            adminPortOutput = CreateStructData();
            AddNumberToStruct(adminPortOutput.structure, "idles".toCharArray(), state.idles);
            clientReqsData = CreateArray();
            for(i = 0d; i < clients.length; i = i + 1d){
                AddNumberToArray(clientReqsData, state.clientReqs[(int)i]);
            }
            AddArrayToStruct(adminPortOutput.structure, "Client Reqs".toCharArray(), clientReqsData);

            adminPortOutputJSON = WriteJSON(adminPortOutput);

            adminPortOutputBytes = UTF16ToUTF8(adminPortOutputJSON);

            ServerSend(adminPort, adminPortOutputBytes);

            FreeData(adminPortOutput);
            delete(adminPortOutputJSON);
            FreeByteArray(adminPortOutputBytes);
        }

        if(!clientReq && !adminPortReq){
            state.idles = state.idles + 1d;
            state.idle = true;
        }else{
            state.idle = false;
        }

        delete(request);
        delete(response);
    }

    public static void delete(Object o) {
        // Java has garbage collection.
    }

}







